@@ -179,7 +179,11 @@ ephy_permissions_manager_get_permission (EphyPermissionsManager *manager,
                                          EphyPermissionType      type,
                                          const char             *origin)
 {
-  GSettings *settings = ephy_permissions_manager_get_settings_for_origin (manager, origin);
+  GSettings *settings;
+
+  g_assert (type != EPHY_PERMISSION_TYPE_ACCESS_WEBCAM_AND_MICROPHONE);
+
+  settings = ephy_permissions_manager_get_settings_for_origin (manager, origin);
   return g_settings_get_enum (settings, permission_type_to_string (type));
 }
 
@@ -187,10 +191,13 @@ static gint
 webkit_security_origin_compare (WebKitSecurityOrigin *a,
                                 WebKitSecurityOrigin *b)
 {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   if (webkit_security_origin_is_opaque (a))
     return -1;
   if (webkit_security_origin_is_opaque (b))
     return 1;
+#pragma GCC diagnostic pop
 
   return g_strcmp0 (webkit_security_origin_get_protocol (a), webkit_security_origin_get_protocol (b)) ||
          g_strcmp0 (webkit_security_origin_get_host (a), webkit_security_origin_get_host (b)) ||
@@ -247,6 +254,8 @@ ephy_permissions_manager_set_permission (EphyPermissionsManager *manager,
   WebKitSecurityOrigin *webkit_origin;
   GSettings *settings;
 
+  g_assert (type != EPHY_PERMISSION_TYPE_ACCESS_WEBCAM_AND_MICROPHONE);
+
   webkit_origin = webkit_security_origin_new_for_uri (origin);
   if (webkit_origin == NULL)
     return;
