@@ -41,6 +41,8 @@ struct _EphySyncService {
   SoupSession *session;
   guint source_id;
 
+  GCancellable *cancellable;
+
   char *user;
   char *crypto_keys;
   GHashTable *secrets;
@@ -463,6 +465,7 @@ ephy_sync_service_fxa_hawk_post (EphySyncService     *self,
   SoupMessage *msg;
   char *url;
   const char *content_type = "application/json; charset=utf-8";
+  g_autofree char *accounts_server = NULL;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   g_assert (endpoint);
@@ -470,7 +473,8 @@ ephy_sync_service_fxa_hawk_post (EphySyncService     *self,
   g_assert (key);
   g_assert (request_body);
 
-  url = g_strdup_printf ("%s/%s", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL, endpoint);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/%s", accounts_server, endpoint);
   msg = soup_message_new (SOUP_METHOD_POST, url);
   soup_message_set_request (msg, content_type, SOUP_MEMORY_COPY,
                             request_body, strlen (request_body));
@@ -500,13 +504,15 @@ ephy_sync_service_fxa_hawk_get (EphySyncService     *self,
   SyncCryptoHawkHeader *header;
   SoupMessage *msg;
   char *url;
+  g_autofree char *accounts_server = NULL;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   g_assert (endpoint);
   g_assert (id);
   g_assert (key);
 
-  url = g_strdup_printf ("%s/%s", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL, endpoint);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/%s", accounts_server, endpoint);
   msg = soup_message_new (SOUP_METHOD_GET, url);
   header = ephy_sync_crypto_hawk_header_new (url, "GET", id, key, key_len, NULL);
   soup_message_headers_append (msg->request_headers, "authorization", header->header);
@@ -611,6 +617,7 @@ ephy_sync_service_verify_certificate (EphySyncService *self,
   const char *email;
   gsize len;
   gboolean retval = FALSE;
+  g_autofree char *accounts_server = NULL;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   g_assert (ephy_sync_service_get_secret (self, secrets[UID]));
@@ -660,7 +667,8 @@ ephy_sync_service_verify_certificate (EphySyncService *self,
     g_warning ("JSON object has missing or invalid 'email' member");
     goto out;
   }
-  uri = soup_uri_new (EPHY_SYNC_FX_ACCOUNTS_SERVER_URL);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  uri = soup_uri_new (accounts_server);
   expected = g_strdup_printf ("%s@%s",
                               ephy_sync_service_get_secret (self, secrets[UID]),
                               soup_uri_get_host (uri));
@@ -681,15 +689,16 @@ out:
 }
 
 static void
-forget_secrets_cb (SecretService *service,
-                   GAsyncResult  *result,
-                   gpointer       user_data)
+forget_secrets_cb (GObject      *source_object,
+                   GAsyncResult *result,
+                   gpointer      user_data)
 {
   GError *error = NULL;
 
-  secret_service_clear_finish (service, result, &error);
+  secret_password_clear_finish (result, &error);
   if (error) {
-    g_warning ("Failed to clear sync secrets: %s", error->message);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Failed to clear sync secrets: %s", error->message);
     g_error_free (error);
   } else {
     LOG ("Successfully cleared sync secrets");
@@ -711,8 +720,8 @@ ephy_sync_service_forget_secrets (EphySyncService *self)
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, user,
                                         NULL);
-  secret_service_clear (NULL, EPHY_SYNC_SECRET_SCHEMA, attributes, NULL,
-                        (GAsyncReadyCallback)forget_secrets_cb, NULL);
+  secret_password_clearv (EPHY_SYNC_SECRET_SCHEMA, attributes, self->cancellable,
+                          (GAsyncReadyCallback)forget_secrets_cb, NULL);
   g_hash_table_remove_all (self->secrets);
 
   g_hash_table_unref (attributes);
@@ -746,6 +755,7 @@ ephy_sync_service_destroy_session (EphySyncService *self,
   char *url;
   const char *content_type = "application/json; charset=utf-8";
   const char *request_body = "{}";
+  g_autofree char *accounts_server = NULL;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   if (!session_token)
@@ -753,7 +763,8 @@ ephy_sync_service_destroy_session (EphySyncService *self,
   g_assert (session_token);
 
   /* This also destroys the device associated with the session token. */
-  url = g_strdup_printf ("%s/session/destroy", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/session/destroy", accounts_server);
   ephy_sync_crypto_derive_session_token (session_token, &token_id,
                                          &req_hmac_key, &tmp);
   token_id_hex = ephy_sync_utils_encode_hex (token_id, 32);
@@ -877,12 +888,14 @@ ephy_sync_service_trade_browserid_assertion (EphySyncService *self)
   char *audience;
   char *assertion;
   char *authorization;
+  g_autofree char *token_server = NULL;
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
   g_assert (self->certificate);
   g_assert (self->key_pair);
 
-  audience = ephy_sync_utils_get_audience (EPHY_SYNC_FX_TOKEN_SERVER_URL);
+  token_server = ephy_sync_utils_get_token_server ();
+  audience = ephy_sync_utils_get_audience (token_server);
   assertion = ephy_sync_crypto_create_assertion (self->certificate, audience,
                                                  300, self->key_pair);
   kb = ephy_sync_utils_decode_hex (ephy_sync_service_get_secret (self, secrets[MASTER_KEY]));
@@ -890,7 +903,7 @@ ephy_sync_service_trade_browserid_assertion (EphySyncService *self)
   client_state = g_strndup (hashed_kb, 32);
   authorization = g_strdup_printf ("BrowserID %s", assertion);
 
-  msg = soup_message_new (SOUP_METHOD_GET, EPHY_SYNC_FX_TOKEN_SERVER_URL);
+  msg = soup_message_new (SOUP_METHOD_GET, token_server);
   /* We need to add the X-Client-State header so that the Token Server will
    * recognize accounts that were previously used to sync Firefox data too.
    */
@@ -1010,6 +1023,9 @@ ephy_sync_service_get_storage_credentials (EphySyncService *self)
 
   /* Derive tokenID and reqHMACkey from sessionToken. */
   session_token = ephy_sync_service_get_secret (self, secrets[SESSION_TOKEN]);
+  if (!session_token)
+    return;
+
   ephy_sync_crypto_derive_session_token (session_token, &token_id,
                                          &req_hmac_key, &tmp);
   token_id_hex = ephy_sync_utils_encode_hex (token_id, 32);
@@ -1670,10 +1686,7 @@ ephy_sync_service_stop_periodical_sync (EphySyncService *self)
 {
   g_assert (EPHY_IS_SYNC_SERVICE (self));
 
-  if (self->source_id != 0) {
-    g_source_remove (self->source_id);
-    self->source_id = 0;
-  }
+  g_clear_handle_id (&self->source_id, g_source_remove);
 }
 
 static void
@@ -1688,7 +1701,7 @@ sync_frequency_changed_cb (GSettings       *settings,
 }
 
 static void
-load_secrets_cb (SecretService   *service,
+load_secrets_cb (GObject         *source_object,
                  GAsyncResult    *result,
                  EphySyncService *self)
 {
@@ -1700,8 +1713,12 @@ load_secrets_cb (SecretService   *service,
   const char *message;
   const char *suggestion;
 
-  res = secret_service_search_finish (service, result, &error);
+  res = secret_password_search_finish (result, &error);
   if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+      g_error_free (error);
+      goto out_no_error;
+    }
     g_warning ("Failed to search for sync secrets: %s", error->message);
     g_error_free (error);
     message = _("Could not find the sync secrets for the current sync user.");
@@ -1769,23 +1786,25 @@ ephy_sync_service_load_secrets (EphySyncService *self)
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, user,
                                         NULL);
-  secret_service_search (NULL, EPHY_SYNC_SECRET_SCHEMA, attributes,
-                         SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
-                         NULL, (GAsyncReadyCallback)load_secrets_cb, self);
+  secret_password_searchv (EPHY_SYNC_SECRET_SCHEMA, attributes,
+                           SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                           self->cancellable, (GAsyncReadyCallback)load_secrets_cb, self);
 
   g_hash_table_unref (attributes);
   g_free (user);
 }
 
 static void
-store_secrets_cb (SecretService   *service,
+store_secrets_cb (GObject         *source_object,
                   GAsyncResult    *result,
                   EphySyncService *self)
 {
   GError *error = NULL;
 
-  secret_service_store_finish (service, result, &error);
+  secret_password_store_finish (result, &error);
   if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      return;
     g_warning ("Failed to store sync secrets: %s", error->message);
     ephy_sync_service_destroy_session (self, NULL);
     g_hash_table_remove_all (self->secrets);
@@ -1807,7 +1826,6 @@ ephy_sync_service_store_secrets (EphySyncService *self)
 {
   JsonNode *node;
   JsonObject *object;
-  SecretValue *secret;
   GHashTable *attributes;
   GHashTableIter iter;
   gpointer key;
@@ -1827,7 +1845,6 @@ ephy_sync_service_store_secrets (EphySyncService *self)
   json_node_set_object (node, object);
   json_string = json_to_string (node, FALSE);
 
-  secret = secret_value_new (json_string, -1, "text/plain");
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, self->user,
                                         NULL);
@@ -1835,13 +1852,12 @@ ephy_sync_service_store_secrets (EphySyncService *self)
   label = g_strdup_printf (_("The sync secrets of %s"), self->user);
 
   LOG ("Storing sync secrets...");
-  secret_service_store (NULL, EPHY_SYNC_SECRET_SCHEMA,
-                        attributes, NULL, label, secret, NULL,
-                        (GAsyncReadyCallback)store_secrets_cb, self);
+  secret_password_storev (EPHY_SYNC_SECRET_SCHEMA,
+                          attributes, NULL, label, json_string, NULL,
+                          (GAsyncReadyCallback)store_secrets_cb, self);
 
   g_free (label);
   g_free (json_string);
-  secret_value_unref (secret);
   g_hash_table_unref (attributes);
   json_object_unref (object);
   json_node_unref (node);
@@ -1950,6 +1966,11 @@ ephy_sync_service_dispose (GObject *object)
   g_clear_object (&self->session);
   g_clear_pointer (&self->secrets, g_hash_table_unref);
 
+  if (self->cancellable) {
+    g_cancellable_cancel (self->cancellable);
+    g_clear_object (&self->cancellable);
+  }
+
   G_OBJECT_CLASS (ephy_sync_service_parent_class)->dispose (object);
 }
 
@@ -1976,6 +1997,7 @@ ephy_sync_service_init (EphySyncService *self)
   self->session = soup_session_new ();
   self->storage_queue = g_queue_new ();
   self->secrets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+  self->cancellable = g_cancellable_new ();
 
   if (ephy_sync_utils_user_is_signed_in ())
     ephy_sync_service_load_secrets (self);
@@ -2411,6 +2433,10 @@ ephy_sync_service_upload_fxa_device (EphySyncService *self)
 
   g_assert (EPHY_IS_SYNC_SERVICE (self));
 
+  session_token = ephy_sync_service_get_secret (self, secrets[SESSION_TOKEN]);
+  if (!session_token)
+    return;
+
   object = json_object_new ();
   device_name = ephy_sync_utils_get_device_name ();
   json_object_set_string_member (object, "name", device_name);
@@ -2429,7 +2455,6 @@ ephy_sync_service_upload_fxa_device (EphySyncService *self)
   json_node_take_object (node, object);
   body = json_to_string (node, FALSE);
 
-  session_token = ephy_sync_service_get_secret (self, secrets[SESSION_TOKEN]);
   ephy_sync_crypto_derive_session_token (session_token, &token_id, &req_hmac_key, &tmp);
   token_id_hex = ephy_sync_utils_encode_hex (token_id, 32);
 
