@@ -37,7 +37,8 @@ struct _EphyActionBarEnd {
   GtkWidget *downloads_revealer;
   GtkWidget *downloads_button;
   GtkWidget *downloads_popover;
-  GtkWidget *downloads_image;
+  GtkWidget *downloads_icon;
+  GtkWidget *downloads_progress;
 
   guint downloads_button_attention_timeout_id;
 };
@@ -96,7 +97,6 @@ download_added_cb (EphyDownloadsManager *manager,
 
   add_attention (action_bar_end);
   gtk_revealer_set_reveal_child (GTK_REVEALER (action_bar_end->downloads_revealer), TRUE);
-  gtk_widget_queue_draw (action_bar_end->downloads_image);
 
   if (gtk_widget_is_visible (GTK_WIDGET (action_bar_end))) {
     gtk_widget_get_allocation (GTK_WIDGET (action_bar_end->downloads_button), &rect);
@@ -189,7 +189,8 @@ download_completed_cb (EphyDownloadsManager *manager,
                        EphyDownload         *download,
                        EphyActionBarEnd     *action_bar_end)
 {
-  begin_complete_theatrics (action_bar_end);
+  if (gtk_widget_get_mapped (GTK_WIDGET (action_bar_end)))
+    begin_complete_theatrics (action_bar_end);
 }
 
 static void
@@ -205,7 +206,9 @@ static void
 downloads_estimated_progress_cb (EphyDownloadsManager *manager,
                                  EphyActionBarEnd     *action_bar_end)
 {
-  gtk_widget_queue_draw (gtk_button_get_image (GTK_BUTTON (action_bar_end->downloads_button)));
+  gdouble fraction = ephy_downloads_manager_get_estimated_progress (manager);
+
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (action_bar_end->downloads_progress), fraction);
 }
 
 static void
@@ -235,7 +238,10 @@ ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
                                         downloads_button);
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarEnd,
-                                        downloads_image);
+                                        downloads_icon);
+  gtk_widget_class_bind_template_child (widget_class,
+                                        EphyActionBarEnd,
+                                        downloads_progress);
 }
 
 static void
@@ -264,6 +270,11 @@ ephy_action_bar_end_init (EphyActionBarEnd *action_bar_end)
     gtk_button_set_image (GTK_BUTTON (action_bar_end->bookmarks_button),
                           gtk_image_new_from_icon_name ("user-bookmarks",
                                                         GTK_ICON_SIZE_LARGE_TOOLBAR));
+    gtk_image_set_from_icon_name (GTK_IMAGE (action_bar_end->downloads_icon),
+                                  "browser-download",
+                                  GTK_ICON_SIZE_LARGE_TOOLBAR);
+    gtk_style_context_add_class (gtk_widget_get_style_context (action_bar_end->downloads_button),
+                                 "flat");
   }
 
   g_signal_connect_object (downloads_manager, "download-added",
